import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='bm-bd-projects')
@click.pass_context
def bm_bd_projects(ctx):
    """Manage ACRCloud BM (Broadcast Monitoring) Database Projects"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )

# ==================== BM Database Projects ====================

@bm_bd_projects.command(name='list')
@click.option('--region', '-r', help='Filter by region (eu-west-1, us-west-2, ap-southeast-1)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_projects(ctx, region, output):
    """List all BM Database projects
    
    Examples:
        acrcloud bm-bd-projects list
        acrcloud bm-bd-projects list --region eu-west-1
    """
    api = ctx.obj['api']
    
    try:
        response = api.list_bm_bd_projects(region=region)
        projects = response.get('data', [])
        
        if output == 'json':
            output_json(response)
        else:
            if not projects:
                click.echo("No BM database projects found.")
                return
                
            headers = ['ID', 'Name', 'Region', 'Status Check', 'Created At']
            rows = []
            
            # The list API returns array directly in data normally
            project_list = projects if isinstance(projects, list) else [projects]
            
            for proj in project_list:
                rows.append([
                    proj.get('id', ''),
                    proj.get('name', ''),
                    proj.get('region', ''),
                    proj.get('status_check', ''),
                    proj.get('created_at', '')
                ])
                
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='create')
@click.option('--name', '-n', required=True, help='Project name')
@click.option('--region', '-r', required=True,
              type=click.Choice(['eu-west-1', 'us-west-2', 'ap-southeast-1']),
              help='Region')
@click.option('--buckets', '-b', required=True, help='Bucket IDs (comma-separated)')
@click.pass_context
def create_project(ctx, name, region, buckets):
    """Create a new BM Database project
    
    Examples:
        acrcloud bm-bd-projects create --name my-db-project --region eu-west-1 --buckets "14661"
    """
    api = ctx.obj['api']
    
    try:
        bucket_ids = [int(b.strip()) for b in buckets.split(',') if b.strip()]
        result = api.create_bm_bd_project(
            name=name,
            region=region,
            buckets=bucket_ids
        )
        click.echo(f"✓ BM Database project '{name}' created successfully!")
        output_json(result)
        
    except ValueError:
        click.echo("Error: Invalid bucket IDs format. Must be comma-separated integers.", err=True)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='update')
@click.argument('project_id', type=int)
@click.option('--name', '-n', help='Project name')
@click.option('--buckets', '-b', help='Bucket IDs (comma-separated)')
@click.pass_context
def update_project(ctx, project_id, name, buckets):
    """Update a BM Database project
    
    Examples:
        acrcloud bm-bd-projects update 12345 --name new-name
    """
    api = ctx.obj['api']
    
    if not name and not buckets:
        click.echo("Error: Please specify what to update (--name or --buckets)", err=True)
        return
        
    try:
        bucket_ids = None
        if buckets:
            bucket_ids = [int(b.strip()) for b in buckets.split(',') if b.strip()]
            
        result = api.update_bm_bd_project(
            project_id=project_id,
            name=name,
            buckets=bucket_ids
        )
        click.echo(f"✓ BM Database project {project_id} updated successfully!")
        output_json(result)
        
    except ValueError:
        click.echo("Error: Invalid bucket IDs format. Must be comma-separated integers.", err=True)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='delete')
@click.argument('project_id', type=int)
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_project(ctx, project_id, yes):
    """Delete a BM Database project
    
    Examples:
        acrcloud bm-bd-projects delete 12345
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete BM Database project {project_id}?"):
            click.echo("Cancelled")
            return
            
    try:
        api.delete_bm_bd_project(project_id)
        click.echo(f"✓ BM Database project {project_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='set-callback')
@click.argument('project_id', type=int)
@click.option('--url', '-u', required=True, help='Callback URL')
@click.option('--send-noresult', type=int, default=0, help='Inform when no content detected (0 or 1)')
@click.option('--result-type', type=int, default=0, help='Callback result type (0: RealTime, 1: Delay)')
@click.pass_context
def set_callback(ctx, project_id, url, send_noresult, result_type):
    """Set result callback URL for BM Database project
    
    Examples:
        acrcloud bm-bd-projects set-callback 12345 --url https://callback.example.com/results
    """
    api = ctx.obj['api']
    
    try:
        api.set_bm_bd_result_callback(
            project_id=project_id,
            result_callback_url=url,
            result_callback_send_noresult=send_noresult,
            result_callback_result_type=result_type
        )
        click.echo(f"✓ Callback URL set successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='set-state-callback')
@click.argument('project_id', type=int)
@click.option('--url', '-u', required=True, help='State notification URL')
@click.pass_context
def set_state_callback(ctx, project_id, url):
    """Set state notification callback for BM Database project
    
    Examples:
        acrcloud bm-bd-projects set-state-callback 12345 --url https://callback.example.com/state
    """
    api = ctx.obj['api']
    
    try:
        api.set_bm_bd_state_notification_callback(
            project_id=project_id,
            state_callback_url=url
        )
        click.echo(f"✓ State notification callback set successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


# ==================== BM Database Channels ====================

@bm_bd_projects.command(name='list-channels')
@click.argument('project_id', type=int)
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--state', '-s', default='All', type=click.Choice(['All', 'Running', 'Timeout', 'Paused', 'Invalid URL', 'Mute', 'Other']), help='Filter by state')
@click.option('--timemap', type=click.Choice(['0', '1']), help='Filter by timemap')
@click.option('--search-type', type=click.Choice(['channel_id', 'channel_name', 'city', 'custom_id', 'mytuner_id']), help='Search field')
@click.option('--search-value', help='Search value')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_channels(ctx, project_id, page, state, timemap, search_type, search_value, output):
    """List channels in BM Database project
    
    Examples:
        acrcloud bm-bd-projects list-channels 12345
        acrcloud bm-bd-projects list-channels 12345 --state Running
    """
    api = ctx.obj['api']
    
    if search_type and not search_value:
        click.echo("Error: --search-value is required when using --search-type", err=True)
        return
        
    try:
        response = api.list_bm_bd_channels(
            project_id=project_id,
            state=state,
            timemap=timemap,
            search_type=search_type,
            search_value=search_value,
            page=page
        )
        channels = response.get('data', [])
        
        if output == 'json':
            output_json(response)
        else:
            if not channels:
                click.echo("No channels found.")
                return
                
            headers = ['ID', 'Name', 'Status Code', 'Language', 'Country']
            rows = []
            
            for ch in channels:
                rows.append([
                    ch.get('id', ''),
                    ch.get('name', ''),
                    ch.get('status_code', ''),
                    ch.get('language', ''),
                    ch.get('country', '')
                ])
                
            output_table(headers, rows)
            
            if response.get('meta'):
                meta = response['meta']
                click.echo(f"\nPage {meta.get('current_page')} of {meta.get('last_page')} (Total: {meta.get('total', 'Unknown')})")
                
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='add-channels')
@click.argument('project_id', type=int)
@click.option('--channels', '-c', required=True, help='Comma-separated channel IDs')
@click.pass_context
def add_channels(ctx, project_id, channels):
    """Add channels to BM Database project
    
    Examples:
        acrcloud bm-bd-projects add-channels 12345 --channels "238766,238767"
    """
    api = ctx.obj['api']
    
    try:
        channel_ids = [int(c.strip()) for c in channels.split(',') if c.strip()]
        result = api.add_bm_bd_channels(project_id=project_id, channels=channel_ids)
        click.echo(f"✓ Channels added successfully!")
        output_json(result)
        
    except ValueError:
        click.echo("Error: Invalid channel IDs format. Must be comma-separated integers.", err=True)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='delete-channels')
@click.argument('project_id', type=int)
@click.option('--channel-ids', '-c', required=True, help='Comma-separated channel IDs')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_channels(ctx, project_id, channel_ids, yes):
    """Delete channels from BM Database project
    
    Examples:
        acrcloud bm-bd-projects delete-channels 12345 --channel-ids "238766"
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete these channels?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_bm_bd_channels(project_id=project_id, channel_ids=channel_ids)
        click.echo(f"✓ Channels deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='set-custom-id')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--custom-id', '-c', required=True, help='Custom ID to set')
@click.pass_context
def set_custom_id(ctx, project_id, channel_id, custom_id):
    """Set custom_id for a channel
    
    Examples:
        acrcloud bm-bd-projects set-custom-id 12345 238766 --custom-id "MyID_1"
    """
    api = ctx.obj['api']
    
    try:
        result = api.set_bm_bd_channel_custom_id(project_id=project_id, channel_id=channel_id, custom_id=custom_id)
        click.echo(f"✓ Custom ID set successfully!")
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


# ==================== BM Database Channels Data ====================

@bm_bd_projects.command(name='channel-state')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--timeoffset', '-t', type=int, help='UTC time offset (e.g., 0)')
@click.option('--start-date', '-s', help='Start date (YYYYMMDD)')
@click.option('--end-date', '-e', help='End date (YYYYMMDD)')
@click.option('--output', '-o', type=click.Choice(['json']), default='json', help='Output format (only json supported)')
@click.pass_context
def channel_state(ctx, project_id, channel_id, timeoffset, start_date, end_date, output):
    """Get the state of the channel
    
    Examples:
        acrcloud bm-bd-projects channel-state 1234 295704 --timeoffset 0 --start-date 20210301 --end-date 20210302
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_channel_state(
            project_id=project_id,
            channel_id=channel_id,
            timeoffset=timeoffset,
            start_date=start_date,
            end_date=end_date
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='channel-results')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--type', '-t', 'result_type', default='day', type=click.Choice(['last', 'day']), help='Get last or day results')
@click.option('--date', '-d', help='Date (YYYYMMDD)')
@click.option('--min-duration', type=int, help='Min duration seconds')
@click.option('--max-duration', type=int, help='Max duration seconds')
@click.option('--isrc-country', help='ISRC country code')
@click.option('--with-false-positive', type=int, help='Return false positives (0 or 1)')
@click.pass_context
def channel_results(ctx, project_id, channel_id, result_type, date, min_duration, max_duration, isrc_country, with_false_positive):
    """Get non-real-time results of channel monitoring
    
    Examples:
        acrcloud bm-bd-projects channel-results 12345 100251 --date 20210107
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_channel_results(
            project_id=project_id,
            channel_id=channel_id,
            result_type=result_type,
            date=date,
            min_duration=min_duration,
            max_duration=max_duration,
            isrc_country=isrc_country,
            with_false_positive=with_false_positive
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='unknown-results')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--date', '-d', required=True, help='Date (YYYYMMDD)')
@click.option('--min-duration', type=int, help='Min duration seconds')
@click.option('--max-duration', type=int, help='Max duration seconds')
@click.pass_context
def unknown_results(ctx, project_id, channel_id, date, min_duration, max_duration):
    """Get unknown music results of channel monitoring
    
    Examples:
        acrcloud bm-bd-projects unknown-results 12345 100251 --date 20210107
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_channel_unknown_results(
            project_id=project_id,
            channel_id=channel_id,
            date=date,
            min_duration=min_duration,
            max_duration=max_duration
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='realtime-results')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.pass_context
def realtime_results(ctx, project_id, channel_id):
    """Get real-time results of channel monitoring
    
    Examples:
        acrcloud bm-bd-projects realtime-results 12345 100251
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_channel_realtime_results(
            project_id=project_id,
            channel_id=channel_id
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='analytics')
@click.argument('project_id', type=int)
@click.option('--stats-type', '-s', required=True, type=click.Choice(['date', 'track', 'artists', 'label', 'channel']), help='Type of data')
@click.option('--result-type', '-r', required=True, type=click.Choice(['music', 'custom']), help='Type of result')
@click.pass_context
def analytics(ctx, project_id, stats_type, result_type):
    """Get analytics data for the last 7 days
    
    Examples:
        acrcloud bm-bd-projects analytics 1234 --stats-type date --result-type music
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_analytics(
            project_id=project_id,
            stats_type=stats_type,
            result_type=result_type
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='user-report')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--data', '-d', required=True, help='JSON string of user result array')
@click.pass_context
def user_report(ctx, project_id, channel_id, data):
    """Insert user results
    
    Examples:
        acrcloud bm-bd-projects user-report 179 100251 --data '[{"from":"api","title":"test","timeoffset":0}]'
    """
    api = ctx.obj['api']
    
    try:
        data_list = json.loads(data)
    except json.JSONDecodeError:
        click.echo("Error: --data must be a valid JSON array", err=True)
        return
    
    try:
        result = api.add_bm_bd_channel_user_report(
            project_id=project_id,
            channel_id=channel_id,
            data=data_list
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@bm_bd_projects.command(name='channel-recording')
@click.argument('project_id', type=int)
@click.argument('channel_id', type=int)
@click.option('--timestamp-utc', '-t', required=True, help='Start time (YYYYmmddHHMMSS)')
@click.option('--played-duration', '-d', required=True, type=int, help='Duration of recording (seconds)')
@click.option('--record-before', type=int, default=0, help='Seconds of recording to add forward')
@click.option('--record-after', type=int, default=0, help='Seconds of recording to add backwards')
@click.pass_context
def channel_recording(ctx, project_id, channel_id, timestamp_utc, played_duration, record_before, record_after):
    """Get the recording of the results
    
    Examples:
        acrcloud bm-bd-projects channel-recording 100079 100123 -t 20210601121314 -d 30
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bm_bd_channel_recording(
            project_id=project_id,
            channel_id=channel_id,
            timestamp_utc=timestamp_utc,
            played_duration=played_duration,
            record_before=record_before,
            record_after=record_after
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



"""
Files command for ACRCloud CLI
"""

import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='files')
@click.pass_context
def files(ctx):
    """Manage audio files in buckets"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )


@files.command(name='list')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', '-n', default=20, help='Results per page')
@click.option('--keyword', '-k', help='Search keyword')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_files(ctx, bucket_id, page, per_page, keyword, output):
    """List files in a bucket
    
    Examples:
        acrcloud buckets files list --bucket-id 12345
        acrcloud buckets files list -b 12345 --keyword "song"
    """
    api = ctx.obj['api']
    
    try:
        result = api.list_files(bucket_id=bucket_id, page=page, 
                                per_page=per_page, keyword=keyword)
        
        if output == 'json':
            output_json(result)
        else:
            files_data = result.get('data', [])
            if not files_data:
                click.echo("No files found")
                return
            
            headers = ['ID', 'ACRID', 'Title', 'Duration', 'State']
            rows = []
            for file in files_data:
                rows.append([
                    file.get('id'),
                    file.get('acr_id', '-')[:20] + '...' if file.get('acr_id') and len(file.get('acr_id')) > 20 else file.get('acr_id', '-'),
                    file.get('title', '-')[:30],
                    f"{float(file.get('duration', 0)):.2f}s" if file.get('duration') else '-',
                    'Active' if file.get('state') == 1 else 'Processing' if file.get('state') == 0 else 'Inactive'
                ])
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='get')
@click.argument('file_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def get_file(ctx, file_id, bucket_id, output):
    """Get file details
    
    Examples:
        acrcloud buckets files get 67890 --bucket-id 12345
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_file(bucket_id=bucket_id, file_id=file_id)
        
        if output == 'json':
            output_json(result)
        else:
            file_data = result.get('data', {})
            if not file_data:
                click.echo("File not found")
                return
            
            click.echo(f"File Details (ID: {file_id}):")
            click.echo("-" * 40)
            click.echo(f"  ACRID: {file_data.get('acr_id')}")
            click.echo(f"  Title: {file_data.get('title', '-')}")
            click.echo(f"  Duration: {file_data.get('duration', '-')}s")
            click.echo(f"  State: {'Active' if file_data.get('state') == 1 else 'Processing' if file_data.get('state') == 0 else 'Inactive'}")
            click.echo(f"  User Defined: {json.dumps(file_data.get('user_defined', {}))}")
            click.echo(f"  Created: {file_data.get('created_at')}")
            click.echo(f"  Updated: {file_data.get('updated_at')}")
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='upload')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--file', '-f', 'file_path', type=click.Path(exists=True), help='Audio or fingerprint file path')
@click.option('--audio-url', '-u', help='Audio file URL (for audio_url type)')
@click.option('--acrid', '-a', help='ACRID from ACRCloud Music Database')
@click.option('--type', '-t', 'data_type', default='audio',
              type=click.Choice(['audio', 'fingerprint', 'audio_url', 'acrid']),
              help='Upload type')
@click.option('--title', '-n', help='File title')
@click.option('--user-defined', '-d', help='User-defined metadata (JSON string)')
@click.pass_context
def upload_file(ctx, bucket_id, file_path, audio_url, acrid, data_type, title, user_defined):
    """Upload an audio file or fingerprint to a bucket
    
    Examples:
        acrcloud buckets files upload --bucket-id 12345 --file audio.mp3
        acrcloud buckets files upload -b 12345 -f fingerprint.fp -t fingerprint
        acrcloud buckets files upload -b 12345 -u https://example.com/audio.mp3 -t audio_url
        acrcloud buckets files upload -b 12345 -a "ACRID123" -t acrid
    """
    api = ctx.obj['api']
    
    user_defined_dict = json.loads(user_defined) if user_defined else None
    
    try:
        result = api.upload_file(
            bucket_id=bucket_id,
            file_path=file_path,
            title=title,
            data_type=data_type,
            audio_url=audio_url,
            acrid=acrid,
            user_defined=user_defined_dict
        )
        
        file_data = result.get('data', {})
        click.echo(f"✓ File uploaded successfully!")
        click.echo(f"  ID: {file_data.get('id')}")
        click.echo(f"  ACRID: {file_data.get('acr_id')}")
        click.echo(f"  Title: {file_data.get('title', '-')}")
        click.echo(f"  Duration: {file_data.get('duration', '-')}s")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='update')
@click.argument('file_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--title', '-n', help='New title')
@click.option('--user-defined', '-d', help='User-defined metadata (JSON string)')
@click.pass_context
def update_file(ctx, file_id, bucket_id, title, user_defined):
    """Update a file
    
    Examples:
        acrcloud buckets files update 67890 --bucket-id 12345 --title "New Title"
    """
    api = ctx.obj['api']
    
    user_defined_dict = json.loads(user_defined) if user_defined else None
    
    try:
        result = api.update_file(
            bucket_id=bucket_id,
            file_id=file_id,
            title=title,
            user_defined=user_defined_dict
        )
        
        click.echo(f"✓ File {file_id} updated successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='delete')
@click.argument('file_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_file(ctx, file_id, bucket_id, yes):
    """Delete a file
    
    Examples:
        acrcloud buckets files delete 67890 --bucket-id 12345
        acrcloud buckets files delete 67890 -b 12345 --yes
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete file {file_id}?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_file(bucket_id=bucket_id, file_id=file_id)
        click.echo(f"✓ File {file_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='delete-batch')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--file-ids', '-i', required=True, help='Comma-separated file IDs')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_files_batch(ctx, bucket_id, file_ids, yes):
    """Delete multiple files
    
    Examples:
        acrcloud buckets files delete-batch --bucket-id 12345 --file-ids "1,2,3"
    """
    api = ctx.obj['api']
    
    ids = [int(i.strip()) for i in file_ids.split(',')]
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete {len(ids)} files?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_files_batch(bucket_id=bucket_id, file_ids=ids)
        click.echo(f"✓ {len(ids)} files deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='move')
@click.option('--bucket-id', '-b', required=True, type=int, help='Source bucket ID')
@click.option('--target-bucket-id', '-t', required=True, type=int, help='Target bucket ID')
@click.option('--file-ids', '-i', required=True, help='Comma-separated file IDs')
@click.pass_context
def move_files(ctx, bucket_id, target_bucket_id, file_ids):
    """Move files to another bucket
    
    Examples:
        acrcloud buckets files move --bucket-id 12345 --target-bucket-id 67890 --file-ids "1,2,3"
    """
    api = ctx.obj['api']
    
    ids = [int(i.strip()) for i in file_ids.split(',')]
    
    try:
        api.move_files(bucket_id=bucket_id, target_bucket_id=target_bucket_id, file_ids=ids)
        click.echo(f"✓ {len(ids)} files moved to bucket {target_bucket_id} successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@files.command(name='dump')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.pass_context
def dump_files(ctx, bucket_id):
    """Dump all files information in a bucket (once per day limit)
    
    Examples:
        acrcloud buckets files dump --bucket-id 12345
    """
    api = ctx.obj['api']
    
    try:
        result = api.dump_files(bucket_id=bucket_id)
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



"""
Buckets command for ACRCloud CLI
"""

import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='buckets')
@click.pass_context
def buckets(ctx):
    """Manage ACRCloud buckets"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )


@buckets.command(name='list')
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', '-n', default=20, help='Results per page')
@click.option('--region', '-r', help='Filter by region (eu-west-1, us-west-2, ap-southeast-1)')
@click.option('--type', '-t', 'bucket_type', help='Filter by type (File, Live, LiveRec, LiveTimeshift)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_buckets(ctx, page, per_page, region, bucket_type, output):
    """List all buckets
    
    Examples:
        acrcloud buckets list
        acrcloud buckets list --region eu-west-1
        acrcloud buckets list --type File --output json
    """
    api = ctx.obj['api']
    
    try:
        result = api.list_buckets(page=page, per_page=per_page, 
                                  region=region, bucket_type=bucket_type)
        
        if output == 'json':
            output_json(result)
        else:
            buckets_data = result.get('data', [])
            if not buckets_data:
                click.echo("No buckets found")
                return
            
            headers = ['ID', 'Name', 'Type', 'Region', 'Files', 'Labels']
            rows = []
            for bucket in buckets_data:
                labels = ', '.join(bucket.get('labels', [])) or '-'
                rows.append([
                    bucket.get('id'),
                    bucket.get('name'),
                    bucket.get('type'),
                    bucket.get('region'),
                    bucket.get('num', 0),
                    labels
                ])
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@buckets.command(name='get')
@click.argument('bucket_id', type=int)
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def get_bucket(ctx, bucket_id, output):
    """Get bucket details
    
    Examples:
        acrcloud buckets get 12345
        acrcloud buckets get 12345 --output json
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_bucket(bucket_id)
        
        if output == 'json':
            output_json(result)
        else:
            bucket = result.get('data', {})
            if not bucket:
                click.echo("Bucket not found")
                return
            
            click.echo(f"Bucket Details (ID: {bucket_id}):")
            click.echo("-" * 40)
            click.echo(f"  Name: {bucket.get('name')}")
            click.echo(f"  Type: {bucket.get('type')}")
            click.echo(f"  Region: {bucket.get('region')}")
            click.echo(f"  Files: {bucket.get('num', 0)}")
            click.echo(f"  Size: {bucket.get('size', 0)}")
            click.echo(f"  Labels: {', '.join(bucket.get('labels', [])) or '-'}")
            click.echo(f"  State: {'Active' if bucket.get('state') == 1 else 'Inactive'}")
            click.echo(f"  Created: {bucket.get('created_at')}")
            click.echo(f"  Updated: {bucket.get('updated_at')}")
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@buckets.command(name='create')
@click.option('--name', '-n', required=True, help='Bucket name')
@click.option('--type', '-t', 'bucket_type', default='File', 
              type=click.Choice(['File', 'Live', 'LiveRec', 'LiveTimeshift']),
              help='Bucket type')
@click.option('--region', '-r', required=True,
              type=click.Choice(['eu-west-1', 'us-west-2', 'ap-southeast-1']),
              help='Region')
@click.option('--net-type', type=click.Choice(['0', '1', '2']), default='1',
              help='Network type (0=public, 1=private, 2=both)')
@click.option('--labels', '-l', help='Labels (comma-separated)')
@click.option('--metadata-template', '-m', help='Metadata template (JSON string)')
@click.pass_context
def create_bucket(ctx, name, bucket_type, region, net_type, labels, metadata_template):
    """Create a new bucket
    
    Examples:
        acrcloud buckets create --name my-bucket --type File --region eu-west-1
        acrcloud buckets create -n music -t File -r ap-southeast-1 -l "Music,Audio"
    """
    api = ctx.obj['api']
    
    labels_list = [l.strip() for l in labels.split(',')] if labels else None
    
    try:
        result = api.create_bucket(
            name=name,
            bucket_type=bucket_type,
            region=region,
            net_type=int(net_type),
            labels=labels_list,
            metadata_template=metadata_template
        )
        
        bucket = result.get('data', {})
        click.echo(f"✓ Bucket created successfully!")
        click.echo(f"  ID: {bucket.get('id')}")
        click.echo(f"  Name: {bucket.get('name')}")
        click.echo(f"  Type: {bucket.get('type')}")
        click.echo(f"  Region: {bucket.get('region')}")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@buckets.command(name='update')
@click.argument('bucket_id', type=int)
@click.option('--name', '-n', help='New bucket name')
@click.option('--labels', '-l', help='Labels (comma-separated)')
@click.option('--metadata-template', '-m', help='Metadata template (JSON string)')
@click.pass_context
def update_bucket(ctx, bucket_id, name, labels, metadata_template):
    """Update a bucket
    
    Examples:
        acrcloud buckets update 12345 --name new-name
        acrcloud buckets update 12345 -l "Music,Pop,Rock"
    """
    api = ctx.obj['api']
    
    labels_list = [l.strip() for l in labels.split(',')] if labels else None
    
    try:
        result = api.update_bucket(
            bucket_id=bucket_id,
            name=name,
            labels=labels_list,
            metadata_template=metadata_template
        )
        
        click.echo(f"✓ Bucket {bucket_id} updated successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@buckets.command(name='delete')
@click.argument('bucket_id', type=int)
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_bucket(ctx, bucket_id, yes):
    """Delete a bucket
    
    Examples:
        acrcloud buckets delete 12345
        acrcloud buckets delete 12345 --yes
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete bucket {bucket_id}?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_bucket(bucket_id)
        click.echo(f"✓ Bucket {bucket_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='ucf-projects')
@click.pass_context
def ucf_projects(ctx):
    """Manage ACRCloud User Custom Content (UCF) Projects"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )

# ==================== UCF Projects ====================

@ucf_projects.command(name='list')
@click.option('--region', '-r', help='Filter by region (eu-west-1, us-west-2, ap-southeast-1)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_projects(ctx, region, output):
    """List all UCF projects"""
    api = ctx.obj['api']
    
    try:
        response = api.list_ucf_projects(region=region)
        projects = response.get('data', [])
        
        if output == 'json':
            output_json(response)
        else:
            if not projects:
                click.echo("No UCF projects found.")
                return
                
            headers = ['ID', 'Name', 'Region', 'Type', 'Created At']
            rows = []
            
            project_list = projects if isinstance(projects, list) else [projects]
            
            for proj in project_list:
                rows.append([
                    proj.get('id', ''),
                    proj.get('name', ''),
                    proj.get('region', ''),
                    proj.get('type', ''),
                    proj.get('created_at', '')
                ])
                
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='create')
@click.option('--name', '-n', required=True, help='Project name')
@click.option('--region', '-r', required=True,
              type=click.Choice(['eu-west-1', 'us-west-2', 'ap-southeast-1']),
              help='Region')
@click.option('--type', '-t', 'project_type', default='BM', type=click.Choice(['BM', 'FILES']), help='Project type (BM or FILES)')
@click.option('--config', '-c', help='JSON string config (e.g. {"days":3,"min_ms":5000,"max_ms":300000})')
@click.pass_context
def create_project(ctx, name, region, project_type, config):
    """Create a new UCF project"""
    api = ctx.obj['api']
    
    conf_dict = None
    if config:
        try:
            conf_dict = json.loads(config)
        except json.JSONDecodeError:
            click.echo("Error: --config must be a valid JSON dictionary", err=True)
            return

    try:
        result = api.create_ucf_project(
            name=name,
            region=region,
            project_type=project_type,
            config=conf_dict
        )
        click.echo(f"✓ UCF project '{name}' created successfully!")
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='update')
@click.argument('project_id', type=int)
@click.option('--name', '-n', help='Project name')
@click.option('--config', '-c', help='JSON string config')
@click.pass_context
def update_project(ctx, project_id, name, config):
    """Update a UCF project"""
    api = ctx.obj['api']
    
    if not name and not config:
        click.echo("Error: Please specify what to update (--name or --config)", err=True)
        return
        
    conf_dict = None
    if config:
        try:
            conf_dict = json.loads(config)
        except json.JSONDecodeError:
            click.echo("Error: --config must be a valid JSON dictionary", err=True)
            return
            
    try:
        result = api.update_ucf_project(
            project_id=project_id,
            name=name,
            config=conf_dict
        )
        click.echo(f"✓ UCF project {project_id} updated successfully!")
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='delete')
@click.argument('project_id', type=int)
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_project(ctx, project_id, yes):
    """Delete a UCF project"""
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete UCF project {project_id}?"):
            click.echo("Cancelled")
            return
            
    try:
        api.delete_ucf_project(project_id)
        click.echo(f"✓ UCF project {project_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


# ==================== UCF BM Streams ====================

@ucf_projects.command(name='list-streams')
@click.argument('project_id', type=int)
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', default=10, help='Items per page')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_streams(ctx, project_id, page, per_page, output):
    """List UCF BM streams"""
    api = ctx.obj['api']
    
    try:
        response = api.list_ucf_streams(
            project_id=project_id,
            page=page,
            per_page=per_page
        )
        streams = response.get('data', [])
        
        if output == 'json':
            output_json(response)
        else:
            if not streams:
                click.echo("No BM streams found.")
                return
                
            headers = ['ID', 'BM Stream ID', 'BM Project ID', 'Stream Name', 'Project Name', 'From']
            rows = []
            
            for st in streams:
                rows.append([
                    st.get('id', ''),
                    st.get('bm_stream_id', ''),
                    st.get('bm_project_id', ''),
                    st.get('bm_stream_name', ''),
                    st.get('bm_project_name', ''),
                    st.get('from', '')
                ])
                
            output_table(headers, rows)
            
            if response.get('meta'):
                meta = response['meta']
                click.echo(f"\nPage {meta.get('current_page')} of {meta.get('last_page')} (Total: {meta.get('total', 'Unknown')})")
                
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='import-streams')
@click.argument('project_id', type=int)
@click.option('--bm-stream-ids', required=True, help='Comma-separated BM stream IDs')
@click.option('--origin-from', required=True, type=click.Choice(['BM-CUSTOM', 'BM-DATABASE']), help='Source of BM streams')
@click.option('--bm-project-id', required=True, type=int, help='BM project ID')
@click.pass_context
def import_streams(ctx, project_id, bm_stream_ids, origin_from, bm_project_id):
    """Import BM streams to UCF project"""
    api = ctx.obj['api']
    
    try:
        stream_ids_list = [s.strip() for s in bm_stream_ids.split(',') if s.strip()]
        result = api.import_ucf_bm_streams(
            project_id=project_id,
            bm_stream_ids=stream_ids_list,
            origin_from=origin_from,
            bm_project_id=bm_project_id
        )
        click.echo(f"✓ BM streams imported successfully!")
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='delete-streams')
@click.argument('project_id', type=int)
@click.option('--stream-ids', '-s', required=True, help='Comma-separated stream IDs')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_streams(ctx, project_id, stream_ids, yes):
    """Delete UCF BM streams"""
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete these streams ({stream_ids})?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_ucf_bm_streams(project_id=project_id, stream_ids=stream_ids)
        click.echo(f"✓ Streams deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


# ==================== UCF Results ====================

@ucf_projects.command(name='list-results')
@click.argument('project_id', type=int)
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', default=10, help='Items per page')
@click.option('--begin-date', help='Begin date (YYYY-MM-DD)')
@click.option('--end-date', help='End date (YYYY-MM-DD)')
@click.option('--sortby', type=click.Choice(['id', 'num', 'duration_ms']), help='Sort column')
@click.option('--order', type=click.Choice(['asc', 'desc']), help='Sort order')
@click.option('--status', help='Result status filter (e.g. 0, 2, 6, 10)')
@click.option('--min-duration', help='Min duration (seconds)')
@click.option('--max-duration', help='Max duration (seconds)')
@click.option('--streams', help='Comma-separated stream IDs')
@click.option('--ucf-id', help='UCF item ID filter')
@click.option('--label', help='Label filter (e.g. 0, 1, 2, 3, 4)')
@click.option('--label-value', help='Label value (e.g. advertisement name)')
@click.pass_context
def list_results(ctx, project_id, page, per_page, begin_date, end_date, sortby, order, status,
                 min_duration, max_duration, streams, ucf_id, label, label_value):
    """List UCF results"""
    api = ctx.obj['api']
    
    try:
        result = api.list_ucf_results(
            project_id=project_id, page=page, per_page=per_page,
            begin_date=begin_date, end_date=end_date, sortby=sortby,
            order=order, status=status, min_duration=min_duration,
            max_duration=max_duration, streams=streams, ucf_id=ucf_id,
            label=label, label_value=label_value
        )
        output_json(result)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='get-result')
@click.argument('project_id', type=int)
@click.argument('ucf_id', type=str)
@click.pass_context
def get_result(ctx, project_id, ucf_id):
    """Get one UCF item by ID or ACRID"""
    api = ctx.obj['api']
    
    try:
        result = api.get_ucf_result(project_id=project_id, ucf_id=ucf_id)
        output_json(result)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='result-details')
@click.argument('project_id', type=int)
@click.argument('ucf_id', type=int)
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', default=10, help='Items per page')
@click.option('--begin-date', help='Begin date (YYYY-MM-DD)')
@click.option('--end-date', help='End date (YYYY-MM-DD)')
@click.pass_context
def result_details(ctx, project_id, ucf_id, page, per_page, begin_date, end_date):
    """Get details of one UCF item"""
    api = ctx.obj['api']
    
    try:
        result = api.get_ucf_result_details(
            project_id=project_id, ucf_id=ucf_id, page=page, per_page=per_page,
            begin_date=begin_date, end_date=end_date
        )
        output_json(result)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='record-url')
@click.argument('project_id', type=int)
@click.argument('ucf_id', type=str)
@click.option('--extend', type=int, default=20, help='Seconds to extend at start and end')
@click.pass_context
def record_url(ctx, project_id, ucf_id, extend):
    """Get the UCF audio record URL"""
    api = ctx.obj['api']
    
    try:
        result = api.get_ucf_record_url(project_id=project_id, ucf_id=ucf_id, extend=extend)
        output_json(result)
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='delete-result')
@click.argument('project_id', type=int)
@click.argument('ucf_id', type=int)
@click.option('--reserved', type=int, default=0, help='0 to delete, 1 to reserve and ignore')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_result(ctx, project_id, ucf_id, reserved, yes):
    """Delete a UCF item or move to reserved queue"""
    api = ctx.obj['api']
    
    action = "reserve" if reserved == 1 else "delete"
    
    if not yes:
        if not confirm_action(f"Are you sure you want to {action} UCF result {ucf_id}?"):
            click.echo("Cancelled")
            return
            
    try:
        api.delete_ucf_item(project_id=project_id, ucf_id=ucf_id, reserved=reserved)
        click.echo(f"✓ UCF result {ucf_id} successfully {action}d!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@ucf_projects.command(name='set-pending')
@click.argument('project_id', type=int)
@click.argument('ucf_id', type=int)
@click.pass_context
def set_pending(ctx, project_id, ucf_id):
    """Make a UCF item status pending"""
    api = ctx.obj['api']
    
    try:
        api.set_ucf_item_pending(project_id=project_id, ucf_id=ucf_id)
        click.echo(f"✓ UCF result {ucf_id} set to pending!")
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



"""
Tests for configuration module
"""

import unittest
import tempfile
import json
import os
from pathlib import Path
from acrcloud_cli.config import Config


class TestConfig(unittest.TestCase):
    """Test configuration management"""
    
    def setUp(self):
        """Set up test fixtures"""
        self.temp_dir = tempfile.TemporaryDirectory()
        self.config_path = Path(self.temp_dir.name) / 'test_config.json'
    
    def tearDown(self):
        """Clean up test fixtures"""
        self.temp_dir.cleanup()
    
    def test_default_config(self):
        """Test default configuration values"""
        config = Config(config_path=self.config_path)
        self.assertEqual(config.get('base_url'), 'https://api-v2.acrcloud.com/api')
        self.assertIsNone(config.get('access_token'))
    
    def test_set_and_get(self):
        """Test setting and getting configuration values"""
        config = Config(config_path=self.config_path)
        config.set('access_token', 'test_token')
        self.assertEqual(config.get('access_token'), 'test_token')
    
    def test_persistence(self):
        """Test configuration persistence"""
        config1 = Config(config_path=self.config_path)
        config1.set('access_token', 'persistent_token')
        
        # Create new config instance with same path
        config2 = Config(config_path=self.config_path)
        self.assertEqual(config2.get('access_token'), 'persistent_token')
    
    def test_delete(self):
        """Test deleting configuration values"""
        config = Config(config_path=self.config_path)
        config.set('temp_key', 'temp_value')
        self.assertEqual(config.get('temp_key'), 'temp_value')
        
        config.delete('temp_key')
        self.assertIsNone(config.get('temp_key'))
    
    def test_list(self):
        """Test listing all configuration values"""
        config = Config(config_path=self.config_path)
        config.set('key1', 'value1')
        config.set('key2', 'value2')
        
        all_config = config.list()
        self.assertIn('key1', all_config)
        self.assertIn('key2', all_config)
        self.assertEqual(all_config['key1'], 'value1')
        self.assertEqual(all_config['key2'], 'value2')


if __name__ == '__main__':
    unittest.main()



"""
Channels command for ACRCloud CLI
"""

import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='channels')
@click.pass_context
def channels(ctx):
    """Manage live channels in buckets"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )


@channels.command(name='list')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', '-n', default=20, help='Results per page')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_channels(ctx, bucket_id, page, per_page, output):
    """List live channels in a bucket
    
    Examples:
        acrcloud buckets channels list --bucket-id 12345
    """
    api = ctx.obj['api']
    
    try:
        result = api.list_channels(bucket_id=bucket_id, page=page, per_page=per_page)
        
        if output == 'json':
            output_json(result)
        else:
            channels_data = result.get('data', [])
            if not channels_data:
                click.echo("No channels found")
                return
            
            headers = ['ID', 'Name', 'URL', 'Record', 'Timeshift']
            rows = []
            for channel in channels_data:
                rows.append([
                    channel.get('id'),
                    channel.get('name', '-')[:25],
                    channel.get('url', '-')[:40],
                    'Yes' if channel.get('record') else 'No',
                    f"{channel.get('timeshift', 0)}h" if channel.get('timeshift') else '-'
                ])
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@channels.command(name='get')
@click.argument('channel_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def get_channel(ctx, channel_id, bucket_id, output):
    """Get channel details
    
    Examples:
        acrcloud buckets channels get 67890 --bucket-id 12345
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_channel(bucket_id=bucket_id, channel_id=channel_id)
        
        if output == 'json':
            output_json(result)
        else:
            channel = result.get('data', {})
            if not channel:
                click.echo("Channel not found")
                return
            
            click.echo(f"Channel Details (ID: {channel_id}):")
            click.echo("-" * 40)
            click.echo(f"  Name: {channel.get('name')}")
            click.echo(f"  URL: {channel.get('url')}")
            click.echo(f"  Record: {'Yes' if channel.get('record') else 'No'}")
            click.echo(f"  Timeshift: {channel.get('timeshift', 0)} hours")
            click.echo(f"  User Defined: {json.dumps(channel.get('user_defined', {}))}")
            click.echo(f"  Created: {channel.get('created_at')}")
            click.echo(f"  Updated: {channel.get('updated_at')}")
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@channels.command(name='create')
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--name', '-n', required=True, help='Channel name')
@click.option('--url', '-u', required=True, help='Stream URL')
@click.option('--record', '-r', type=int, help='Record duration in hours')
@click.option('--timeshift', '-t', type=int, help='Timeshift duration in hours')
@click.option('--user-defined', '-d', help='User-defined metadata (JSON string)')
@click.pass_context
def create_channel(ctx, bucket_id, name, url, record, timeshift, user_defined):
    """Create a live channel
    
    Examples:
        acrcloud buckets channels create --bucket-id 12345 --name "Radio One" --url "http://stream.example.com/radio"
        acrcloud buckets channels create -b 12345 -n "TV Channel" -u "http://tv.example.com/stream" -r 24 -t 72
    """
    api = ctx.obj['api']
    
    user_defined_dict = json.loads(user_defined) if user_defined else None
    
    try:
        result = api.create_channel(
            bucket_id=bucket_id,
            name=name,
            url=url,
            record=record,
            timeshift=timeshift,
            user_defined=user_defined_dict
        )
        
        channel = result.get('data', {})
        click.echo(f"✓ Channel created successfully!")
        click.echo(f"  ID: {channel.get('id')}")
        click.echo(f"  Name: {channel.get('name')}")
        click.echo(f"  URL: {channel.get('url')}")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@channels.command(name='update')
@click.argument('channel_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--name', '-n', help='New channel name')
@click.option('--url', '-u', help='New stream URL')
@click.option('--record', '-r', type=int, help='Record duration in hours')
@click.option('--timeshift', '-t', type=int, help='Timeshift duration in hours')
@click.pass_context
def update_channel(ctx, channel_id, bucket_id, name, url, record, timeshift):
    """Update a channel
    
    Examples:
        acrcloud buckets channels update 67890 --bucket-id 12345 --name "New Name"
    """
    api = ctx.obj['api']
    
    try:
        result = api.update_channel(
            bucket_id=bucket_id,
            channel_id=channel_id,
            name=name,
            url=url,
            record=record,
            timeshift=timeshift
        )
        
        click.echo(f"✓ Channel {channel_id} updated successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@channels.command(name='delete')
@click.argument('channel_id', type=int)
@click.option('--bucket-id', '-b', required=True, type=int, help='Bucket ID')
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_channel(ctx, channel_id, bucket_id, yes):
    """Delete a channel
    
    Examples:
        acrcloud buckets channels delete 67890 --bucket-id 12345
        acrcloud buckets channels delete 67890 -b 12345 --yes
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete channel {channel_id}?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_channel(bucket_id=bucket_id, channel_id=channel_id)
        click.echo(f"✓ Channel {channel_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



"""
Config command for ACRCloud CLI
"""

import click
from ..config import Config


@click.group(name='config')
def config():
    """Manage ACRCloud CLI configuration"""
    pass


@config.command(name='set')
@click.argument('key')
@click.argument('value')
@click.option('--config-path', '-c', type=click.Path(), 
              help='Path to config file')
def set_config(key, value, config_path):
    """Set a configuration value
    
    Examples:
        acrcloud config set access_token YOUR_ACCESS_TOKEN
        acrcloud config set base_url https://api-v2.acrcloud.com/api
    """
    cfg = Config(config_path=config_path)
    cfg.set(key, value)
    click.echo(f"✓ Set {key} = {value}")


@config.command(name='get')
@click.argument('key')
@click.option('--config-path', '-c', type=click.Path(),
              help='Path to config file')
def get_config(key, config_path):
    """Get a configuration value
    
    Examples:
        acrcloud config get access_token
    """
    cfg = Config(config_path=config_path)
    value = cfg.get(key)
    if value:
        click.echo(value)
    else:
        click.echo(f"Key '{key}' not found")


@config.command(name='list')
@click.option('--config-path', '-c', type=click.Path(),
              help='Path to config file')
def list_config(config_path):
    """List all configuration values"""
    cfg = Config(config_path=config_path)
    config_data = cfg.list()
    
    if not config_data:
        click.echo("No configuration found")
        return
    
    click.echo("Configuration:")
    click.echo("-" * 40)
    for key, value in config_data.items():
        # Mask sensitive values
        if key in ['access_token', 'secret'] and value:
            display_value = value[:10] + "..." if len(value) > 10 else value
        else:
            display_value = value
        click.echo(f"  {key}: {display_value}")


@config.command(name='delete')
@click.argument('key')
@click.option('--config-path', '-c', type=click.Path(),
              help='Path to config file')
def delete_config(key, config_path):
    """Delete a configuration value
    
    Examples:
        acrcloud config delete access_token
    """
    cfg = Config(config_path=config_path)
    cfg.delete(key)
    click.echo(f"✓ Deleted {key}")



"""
Configuration management for ACRCloud CLI
"""

import json
import os
from pathlib import Path


class Config:
    """Manage ACRCloud CLI configuration"""
    
    DEFAULT_CONFIG = {
        'base_url': 'https://api-v2.acrcloud.com/api',
        'access_token': None
    }
    
    def __init__(self, config_path=None):
        if config_path:
            self.config_path = Path(config_path)
        else:
            # Default config location: ~/.acrcloud/config.json
            self.config_path = Path.home() / '.acrcloud' / 'config.json'
        
        self._config = self._load()
    
    def _load(self):
        """Load configuration from file"""
        if self.config_path.exists():
            try:
                with open(self.config_path, 'r') as f:
                    return {**self.DEFAULT_CONFIG, **json.load(f)}
            except (json.JSONDecodeError, IOError) as e:
                print(f"Warning: Could not load config file: {e}")
                return self.DEFAULT_CONFIG.copy()
        return self.DEFAULT_CONFIG.copy()
    
    def save(self):
        """Save configuration to file"""
        self.config_path.parent.mkdir(parents=True, exist_ok=True)
        with open(self.config_path, 'w') as f:
            json.dump(self._config, f, indent=2)
    
    def get(self, key, default=None):
        """Get configuration value"""
        return self._config.get(key, default)
    
    def set(self, key, value):
        """Set configuration value"""
        self._config[key] = value
        self.save()
    
    def delete(self, key):
        """Delete configuration key"""
        if key in self._config:
            del self._config[key]
            self.save()
    
    def list(self):
        """List all configuration"""
        return self._config.copy()



"""
ACRCloud API Client
"""

import requests
import json
import os
import mimetypes
from pathlib import Path
from typing import Optional, Dict, Any, List
from datetime import datetime, timedelta


class ContainerCache:
    """Local cache for container info to reduce API calls"""
    
    def __init__(self, ttl_minutes: int = 30):
        self._cache: Dict[int, Dict] = {}
        self._timestamps: Dict[int, datetime] = {}
        self._ttl = timedelta(minutes=ttl_minutes)
        self._cache_file = Path.home() / '.acrcloud' / 'container_cache.json'
        self._cache_file.parent.mkdir(parents=True, exist_ok=True)
        self._load_from_disk()
    
    def _load_from_disk(self):
        """Load cache from disk"""
        if self._cache_file.exists():
            try:
                with open(self._cache_file, 'r') as f:
                    data = json.load(f)
                    for cid, info in data.get('containers', {}).items():
                        cid_int = int(cid)
                        self._cache[cid_int] = info
                        ts = datetime.fromisoformat(info.get('_cached_at', ''))
                        self._timestamps[cid_int] = ts
            except Exception:
                pass
    
    def _save_to_disk(self):
        """Save cache to disk"""
        try:
            data = {
                'containers': {
                    str(cid): info for cid, info in self._cache.items()
                }
            }
            with open(self._cache_file, 'w') as f:
                json.dump(data, f)
        except Exception:
            pass
    
    def get(self, container_id: int) -> Optional[Dict]:
        """Get cached container info, returns None if expired or not found"""
        ts = self._timestamps.get(container_id)
        if ts and datetime.now() - ts < self._ttl:
            return self._cache.get(container_id)
        # Expired, remove
        self._cache.pop(container_id, None)
        self._timestamps.pop(container_id, None)
        return None
    
    def set(self, container_id: int, info: Dict):
        """Cache container info"""
        info['_cached_at'] = datetime.now().isoformat()
        self._cache[container_id] = info
        self._timestamps[container_id] = datetime.now()
        self._save_to_disk()
    
    def invalidate(self, container_id: Optional[int] = None):
        """Invalidate cache"""
        if container_id:
            self._cache.pop(container_id, None)
            self._timestamps.pop(container_id, None)
        else:
            self._cache.clear()
            self._timestamps.clear()
        self._save_to_disk()


class ACRCloudAPI:
    """ACRCloud Console API Client"""
    
    def __init__(self, access_token: str, base_url: str = 'https://api-v2.acrcloud.com/api'):
        self.access_token = access_token
        self.base_url = base_url.rstrip('/')
        self.session = requests.Session()
        self.session.headers.update({
            'Authorization': f'Bearer {access_token}',
            'Accept': 'application/json',
            'Content-Type': 'application/json'
        })
        self._container_cache = ContainerCache()
    
    def _request(self, method: str, endpoint: str, base_url: Optional[str] = None, **kwargs) -> Dict[str, Any]:
        """Make HTTP request to API"""
        url = f"{base_url if base_url else self.base_url}{endpoint}"
        
        headers = kwargs.get('headers', {})
        
        # Handle files separately for multipart/form-data
        if 'files' in kwargs:
            # Explicitly suppress session's Content-Type for multipart
            if 'Content-Type' not in headers:
                headers['Content-Type'] = None
            response = self.session.request(method, url, headers=headers, **kwargs)
        else:
            response = self.session.request(method, url, **kwargs)
        
        try:
            response.raise_for_status()
            return response.json() if response.content else {}
        except requests.exceptions.HTTPError as e:
            error_msg = f"HTTP Error {response.status_code}"
            try:
                error_data = response.json()
                if 'message' in error_data:
                    error_msg = f"{error_msg}: {error_data['message']}"
                elif 'error' in error_data:
                    error_msg = f"{error_msg}: {error_data['error']}"
            except:
                error_msg = f"{error_msg}: {response.text}"
            raise APIError(error_msg, response.status_code)
        except json.JSONDecodeError:
            return {'raw_response': response.text}
    
    # ==================== Buckets ====================
    
    def list_buckets(self, page: int = 1, per_page: int = 20, 
                     region: Optional[str] = None, 
                     bucket_type: Optional[str] = None) -> Dict:
        """List all buckets"""
        params = {'page': page, 'per_page': per_page}
        if region:
            params['region'] = region
        if bucket_type:
            params['type'] = bucket_type
        return self._request('GET', '/buckets', params=params)
    
    def get_bucket(self, bucket_id: int) -> Dict:
        """Get a specific bucket"""
        return self._request('GET', f'/buckets/{bucket_id}')
    
    def create_bucket(self, name: str, bucket_type: str, region: str,
                      net_type: int = 1, labels: Optional[List[str]] = None,
                      metadata_template: Optional[str] = None) -> Dict:
        """Create a new bucket"""
        data = {
            'name': name,
            'type': bucket_type,
            'region': region,
            'net_type': net_type
        }
        if labels:
            data['labels'] = labels
        if metadata_template:
            data['metadata_template'] = metadata_template
        return self._request('POST', '/buckets', json=data)
    
    def update_bucket(self, bucket_id: int, name: Optional[str] = None,
                      labels: Optional[List[str]] = None,
                      metadata_template: Optional[str] = None) -> Dict:
        """Update a bucket"""
        data = {}
        if name:
            data['name'] = name
        if labels:
            data['labels'] = labels
        if metadata_template:
            data['metadata_template'] = metadata_template
        return self._request('PUT', f'/buckets/{bucket_id}', json=data)
    
    def delete_bucket(self, bucket_id: int) -> Dict:
        """Delete a bucket"""
        return self._request('DELETE', f'/buckets/{bucket_id}')
    
    # ==================== Audio Files ====================
    
    def list_files(self, bucket_id: int, page: int = 1, 
                   per_page: int = 20, keyword: Optional[str] = None) -> Dict:
        """List files in a bucket"""
        params = {'page': page, 'per_page': per_page}
        if keyword:
            params['keyword'] = keyword
        return self._request('GET', f'/buckets/{bucket_id}/files', params=params)
    
    def get_file(self, bucket_id: int, file_id: int) -> Dict:
        """Get a specific file"""
        return self._request('GET', f'/buckets/{bucket_id}/files/{file_id}')
    
    def upload_file(self, bucket_id: int, file_path: Optional[str] = None,
                    title: Optional[str] = None, data_type: str = 'audio',
                    audio_url: Optional[str] = None, acrid: Optional[str] = None,
                    user_defined: Optional[Dict] = None) -> Dict:
        """Upload an audio file or fingerprint to a bucket"""
        data = {'data_type': data_type}
        
        if title:
            data['title'] = title
        if user_defined:
            data['user_defined'] = json.dumps(user_defined)
        
        files = None
        
        if data_type == 'audio' and file_path:
            files = {'file': open(file_path, 'rb')}
        elif data_type == 'fingerprint' and file_path:
            files = {'file': open(file_path, 'rb')}
        elif data_type == 'audio_url' and audio_url:
            data['url'] = audio_url
        elif data_type == 'acrid' and acrid:
            data['acrid'] = acrid
        
        try:
            if files:
                return self._request('POST', f'/buckets/{bucket_id}/files', data=data, files=files)
            else:
                return self._request('POST', f'/buckets/{bucket_id}/files', json=data)
        finally:
            if files:
                for f in files.values():
                    f.close()
    
    def update_file(self, bucket_id: int, file_id: int, 
                    title: Optional[str] = None,
                    user_defined: Optional[Dict] = None) -> Dict:
        """Update a file"""
        data = {}
        if title:
            data['title'] = title
        if user_defined:
            data['user_defined'] = json.dumps(user_defined)
        return self._request('PUT', f'/buckets/{bucket_id}/files/{file_id}', json=data)
    
    def delete_file(self, bucket_id: int, file_id: int) -> Dict:
        """Delete a file"""
        return self._request('DELETE', f'/buckets/{bucket_id}/files/{file_id}')
    
    def delete_files_batch(self, bucket_id: int, file_ids: List[int]) -> Dict:
        """Delete multiple files"""
        return self._request('DELETE', f'/buckets/{bucket_id}/files', json={'ids': file_ids})
    
    def move_files(self, bucket_id: int, target_bucket_id: int, 
                   file_ids: List[int]) -> Dict:
        """Move files to another bucket"""
        data = {
            'target_bucket_id': target_bucket_id,
            'ids': file_ids
        }
        return self._request('POST', f'/buckets/{bucket_id}/files/move', json=data)
    
    def dump_files(self, bucket_id: int) -> Dict:
        """Dump all files information in a bucket (once per day limit)"""
        return self._request('GET', f'/buckets/{bucket_id}/files/dump')
    
    # ==================== Live Channels ====================
    
    def list_channels(self, bucket_id: int, page: int = 1, 
                      per_page: int = 20) -> Dict:
        """List live channels in a bucket"""
        params = {'page': page, 'per_page': per_page}
        return self._request('GET', f'/buckets/{bucket_id}/channels', params=params)
    
    def get_channel(self, bucket_id: int, channel_id: int) -> Dict:
        """Get a specific channel"""
        return self._request('GET', f'/buckets/{bucket_id}/channels/{channel_id}')
    
    def create_channel(self, bucket_id: int, name: str, url: str,
                       record: Optional[int] = None, 
                       timeshift: Optional[int] = None,
                       user_defined: Optional[Dict] = None) -> Dict:
        """Create a live channel"""
        data = {'name': name, 'url': url}
        if record is not None:
            data['record'] = record
        if timeshift is not None:
            data['timeshift'] = timeshift
        if user_defined:
            data['user_defined'] = user_defined
        return self._request('POST', f'/buckets/{bucket_id}/channels', json=data)
    
    def update_channel(self, bucket_id: int, channel_id: int,
                       name: Optional[str] = None, url: Optional[str] = None,
                       record: Optional[int] = None,
                       timeshift: Optional[int] = None) -> Dict:
        """Update a channel"""
        data = {}
        if name:
            data['name'] = name
        if url:
            data['url'] = url
        if record is not None:
            data['record'] = record
        if timeshift is not None:
            data['timeshift'] = timeshift
        return self._request('PUT', f'/buckets/{bucket_id}/channels/{channel_id}', json=data)
    
    def delete_channel(self, bucket_id: int, channel_id: int) -> Dict:
        """Delete a channel"""
        return self._request('DELETE', f'/buckets/{bucket_id}/channels/{channel_id}')
    
    # ==================== Base Projects ====================
    
    def list_projects(self, page: int = 1, per_page: int = 20) -> Dict:
        """List all base projects"""
        params = {'page': page, 'per_page': per_page}
        return self._request('GET', '/base-projects', params=params)
    
    def get_project(self, project_id: int) -> Dict:
        """Get a specific project"""
        return self._request('GET', f'/base-projects/{project_id}')
    
    def create_project(self, name: str, project_type: str, region: str,
                       buckets: List[int], audio_type: str = 'linein',
                       external_ids: Optional[str] = None) -> Dict:
        """Create a new recognition project"""
        data = {
            'name': name,
            'type': project_type,
            'region': region,
            'buckets': buckets,
            'audio_type': audio_type
        }
        if external_ids:
            data['external_ids'] = external_ids
        return self._request('POST', '/base-projects', json=data)
    
    def update_project(self, project_id: int, name: Optional[str] = None,
                       buckets: Optional[List[int]] = None,
                       audio_type: Optional[str] = None) -> Dict:
        """Update a project"""
        data = {}
        if name:
            data['name'] = name
        if buckets:
            data['buckets'] = buckets
        if audio_type:
            data['audio_type'] = audio_type
        return self._request('PUT', f'/base-projects/{project_id}', json=data)
    
    def delete_project(self, project_id: int) -> Dict:
        """Delete a project"""
        return self._request('DELETE', f'/base-projects/{project_id}')
    
    def get_project_bucket_status(self, project_id: int) -> Dict:
        """Get the status of project's buckets"""
        return self._request('GET', f'/base-projects/{project_id}/bucket-status')
    
    def get_project_statistics(self, project_id: int, 
                               start_date: Optional[str] = None,
                               end_date: Optional[str] = None) -> Dict:
        """Get project statistics"""
        params = {}
        if start_date:
            params['start_date'] = start_date
        if end_date:
            params['end_date'] = end_date
        return self._request('GET', f'/base-projects/{project_id}/statistics', params=params)
    
    # ==================== File Scanning (FS Containers) ====================
    
    def list_fs_containers(self, page: int = 1, per_page: int = 20,
                           region: Optional[str] = None,
                           name: Optional[str] = None) -> Dict:
        """List all file scanning containers"""
        params = {'page': page, 'per_page': per_page}
        if region:
            params['region'] = region
        if name:
            params['name'] = name
        return self._request('GET', '/fs-containers', params=params)
    
    def get_fs_container(self, container_id: int) -> Dict:
        """Get a specific file scanning container"""
        result = self._request('GET', f'/fs-containers/{container_id}')
        # Cache the result
        self._container_cache.set(container_id, result)
        return result
    
    def create_fs_container(self, name: str, region: str, audio_type: str,
                            buckets: List, engine: int, policy: Dict,
                            callback_url: Optional[str] = None,
                            deepright: Optional[bool] = None,
                            music_detection: Optional[bool] = None,
                            ai_detection: Optional[bool] = None) -> Dict:
        """Create a new file scanning container"""
        data = {
            'name': name,
            'region': region,
            'audio_type': audio_type,
            'buckets': buckets,
            'engine': engine,
            'policy': policy
        }
        if callback_url:
            data['callback_url'] = callback_url
        if deepright is not None:
            data['deepright'] = deepright
        if music_detection is not None:
            data['music_detection'] = music_detection
        if ai_detection is not None:
            data['ai_detection'] = ai_detection
        return self._request('POST', '/fs-containers', json=data)
    
    def update_fs_container(self, container_id: int, name: Optional[str] = None,
                            audio_type: Optional[str] = None,
                            buckets: Optional[List] = None,
                            engine: Optional[int] = None,
                            policy: Optional[Dict] = None,
                            callback_url: Optional[str] = None,
                            deepright: Optional[bool] = None,
                            music_detection: Optional[bool] = None,
                            ai_detection: Optional[bool] = None) -> Dict:
        """Update a file scanning container"""
        data = {}
        if name:
            data['name'] = name
        if audio_type:
            data['audio_type'] = audio_type
        if buckets:
            data['buckets'] = buckets
        if engine is not None:
            data['engine'] = engine
        if policy:
            data['policy'] = policy
        if callback_url:
            data['callback_url'] = callback_url
        if deepright is not None:
            data['deepright'] = deepright
        if music_detection is not None:
            data['music_detection'] = music_detection
        if ai_detection is not None:
            data['ai_detection'] = ai_detection
        return self._request('PUT', f'/fs-containers/{container_id}', json=data)
    
    def delete_fs_container(self, container_id: int) -> Dict:
        """Delete a file scanning container"""
        return self._request('DELETE', f'/fs-containers/{container_id}')
    
    # ==================== File Scanning (FsFiles) ====================
    
    def _get_fs_base_url(self, region: str) -> str:
        """Get region-specific base URL for file scanning"""
        return f'https://api-{region}.acrcloud.com/api'
    
    def _resolve_fs_region(self, container_id: int) -> str:
        """Resolve region by fetching container info from cache or main API"""
        # Try cache first
        cached = self._container_cache.get(container_id)
        if cached:
            region = cached.get('data', {}).get('region')
            if region:
                return region
        
        # Fetch from API
        container = self.get_fs_container(container_id)
        # Cache the result
        self._container_cache.set(container_id, container)
        return container.get('data', {}).get('region')
    
    def list_fs_files(self, container_id: int, region: Optional[str] = None,
                      page: int = 1, per_page: int = 20,
                      search: Optional[str] = None, with_result: Optional[int] = None,
                      state: Optional[str] = None) -> Dict:
        """List files in a file scanning container"""
        if not region:
            region = self._resolve_fs_region(container_id)
        base_url = self._get_fs_base_url(region)
        params = {'page': page, 'per_page': per_page}
        if search:
            params['search'] = search
        if with_result is not None:
            params['with_result'] = with_result
        if state:
            params['state'] = state
        return self._request('GET', f'/fs-containers/{container_id}/files', 
                            base_url=base_url, params=params)
    
    def get_fs_file(self, container_id: int, file_id: str,
                    region: Optional[str] = None) -> Dict:
        """Get a specific file scanning file"""
        if not region:
            region = self._resolve_fs_region(container_id)
        return self._request('GET', f'/fs-containers/{container_id}/files/{file_id}',
                            base_url=self._get_fs_base_url(region))
    
    def _get_presigned_upload_url(self, container_id: int, region: str,
                                  filename: Optional[str] = None,
                                  content_type: Optional[str] = None) -> Dict:
        """Get a presigned S3 upload URL for file scanning"""
        base_url = self._get_fs_base_url(region)
        params = {}
        if filename:
            params['filename'] = filename
        if content_type:
            params['content_type'] = content_type
        return self._request('GET', f'/fs-containers/{container_id}/presigned-upload',
                            base_url=base_url, params=params)

    def upload_fs_file(self, container_id: int,
                       file_path: Optional[str] = None,
                       audio_url: Optional[str] = None,
                       data_type: str = 'audio',
                       name: Optional[str] = None,
                       region: Optional[str] = None) -> Dict:
        """Upload a file to file scanning container

        For audio/fingerprint types with a local file, uses presigned URL flow:
        1. GET presigned upload URL
        2. PUT file to S3
        3. POST file metadata to create the record

        For other types (audio_url, etc.), uses direct JSON POST.
        """
        if not region:
            region = self._resolve_fs_region(container_id)
        base_url = self._get_fs_base_url(region)

        # Use presigned URL flow for audio/fingerprint with local file
        if data_type in ('audio', 'fingerprint') and file_path:
            filename = name or os.path.basename(file_path)
            content_type = mimetypes.guess_type(file_path)[0] or 'audio/mpeg'

            # Step 1: Get presigned upload URL
            presigned_result = self._get_presigned_upload_url(
                container_id, region,
                filename=filename,
                content_type=content_type
            )
            presigned_data = presigned_result.get('data', {})
            presigned_url = presigned_data['presigned_url']
            s3_key = presigned_data['key']
            s3_headers = presigned_data.get('headers', {})

            # Step 2: PUT file to S3
            with open(file_path, 'rb') as f:
                upload_resp = requests.request(
                    presigned_data.get('method', 'PUT'),
                    presigned_url,
                    data=f,
                    headers=s3_headers
                )
                upload_resp.raise_for_status()

            # Step 3: POST file record with S3 key
            data = {
                'data_type': data_type,
                'key': s3_key,
                'filename': filename,
            }
            if name:
                data['name'] = name

            return self._request('POST', f'/fs-containers/{container_id}/files',
                                base_url=base_url, json=data)

        # Non-file types: direct JSON POST
        data = {'data_type': data_type}
        if name:
            data['name'] = name

        if data_type == 'audio_url' and audio_url:
            data['url'] = audio_url

        return self._request('POST', f'/fs-containers/{container_id}/files',
                            base_url=base_url, json=data)
    
    def delete_fs_files(self, container_id: int, file_ids: str,
                        region: Optional[str] = None) -> Dict:
        """Delete files from file scanning container"""
        if not region:
            region = self._resolve_fs_region(container_id)
        return self._request('DELETE', f'/fs-containers/{container_id}/files/{file_ids}',
                            base_url=self._get_fs_base_url(region))
    
    def rescan_fs_files(self, container_id: int, file_ids: str,
                        region: Optional[str] = None) -> Dict:
        """Rescan files in file scanning container"""
        if not region:
            region = self._resolve_fs_region(container_id)
        return self._request('PUT', f'/fs-containers/{container_id}/files/{file_ids}/rescan',
                            base_url=self._get_fs_base_url(region))
    
    # ==================== BM Custom Streams Projects ====================
    
    def list_bm_cs_projects(self, page: int = 1, per_page: int = 20,
                            region: Optional[str] = None,
                            types: Optional[str] = None) -> Dict:
        """List all BM custom streams projects"""
        params = {'page': page, 'per_page': per_page}
        if region:
            params['region'] = region
        if types:
            params['types'] = types
        return self._request('GET', '/bm-cs-projects', params=params)
    
    def get_bm_cs_project(self, project_id: int) -> Dict:
        """Get a specific BM custom streams project"""
        return self._request('GET', f'/bm-cs-projects/{project_id}')
    
    def create_bm_cs_project(self, name: str, region: str, buckets: List,
                             project_type: str = 'BM-ACRC',
                             external_ids: Optional[str] = None,
                             metadata_template: Optional[str] = None) -> Dict:
        """Create a new BM custom streams project"""
        data = {
            'name': name,
            'region': region,
            'buckets': buckets,
            'type': project_type
        }
        if external_ids:
            data['external_ids'] = external_ids
        if metadata_template:
            data['metadata_template'] = metadata_template
        return self._request('POST', '/bm-cs-projects', json=data)
    
    def update_bm_cs_project(self, project_id: int, name: Optional[str] = None,
                             buckets: Optional[List] = None,
                             external_ids: Optional[List] = None,
                             metadata_template: Optional[str] = None) -> Dict:
        """Update a BM custom streams project"""
        data = {}
        if name:
            data['name'] = name
        if buckets:
            data['buckets'] = buckets
        if external_ids:
            data['external_ids'] = external_ids
        if metadata_template:
            data['metadata_template'] = metadata_template
        return self._request('PUT', f'/bm-cs-projects/{project_id}', json=data)
    
    def delete_bm_cs_project(self, project_id: int) -> Dict:
        """Delete a BM custom streams project"""
        return self._request('DELETE', f'/bm-cs-projects/{project_id}')
    
    def set_bm_cs_result_callback(self, project_id: int, callback_url: str) -> Dict:
        """Set result callback URL for BM custom streams project"""
        data = {'result_callback_url': callback_url}
        return self._request('POST', f'/bm-cs-projects/{project_id}/result-callback', json=data)
    
    def set_bm_cs_state_notification_callback(self, project_id: int, 
                                              email: Optional[str] = None,
                                              frequency: Optional[int] = None,
                                              url: Optional[str] = None) -> Dict:
        """Set state notification callback for BM custom streams project"""
        data = {}
        if email:
            data['state_notification_email'] = email
        if frequency is not None:
            data['state_notification_email_frequency'] = frequency  # 0:High, 1:Low, 2:None
        if url:
            data['state_notification_url'] = url
        return self._request('POST', f'/bm-cs-projects/{project_id}/state-notification', json=data)
    
    # ==================== BM Custom Streams ====================
    
    def list_bm_streams(self, project_id: int, page: int = 1, per_page: int = 50,
                        timemap: Optional[int] = None,
                        state: Optional[str] = None,
                        search_value: Optional[str] = None,
                        sort: Optional[str] = None,
                        order: Optional[str] = None) -> Dict:
        """List streams in a BM custom streams project"""
        params = {'page': page, 'per_page': per_page}
        if timemap is not None:
            params['timemap'] = timemap
        if state:
            params['state'] = state
        if search_value:
            params['search_value'] = search_value
        if sort:
            params['sort'] = sort
        if order:
            params['order'] = order
        return self._request('GET', f'/bm-cs-projects/{project_id}/streams', params=params)
    
    def add_bm_stream(self, project_id: int, stream_urls: List[str], name: str,
                      config_id: int, user_defined: Optional[str] = None) -> Dict:
        """Add a stream to BM custom streams project"""
        data = {
            'stream_urls': stream_urls,
            'name': name,
            'config_id': config_id
        }
        if user_defined:
            data['user_defined'] = user_defined
        return self._request('POST', f'/bm-cs-projects/{project_id}/streams', json=data)
    
    def update_bm_stream(self, project_id: int, stream_id: str,
                         stream_urls: Optional[List[str]] = None,
                         name: Optional[str] = None,
                         config_id: Optional[int] = None) -> Dict:
        """Update a stream in BM custom streams project"""
        data = {}
        if stream_urls:
            data['stream_urls'] = stream_urls
        if name:
            data['name'] = name
        if config_id:
            data['config_id'] = config_id
        return self._request('PUT', f'/bm-cs-projects/{project_id}/streams/{stream_id}', json=data)
    
    def delete_bm_streams(self, project_id: int, stream_ids: str) -> Dict:
        """Delete streams from BM custom streams project"""
        return self._request('DELETE', f'/bm-cs-projects/{project_id}/streams/{stream_ids}')
    
    def pause_bm_streams(self, project_id: int, stream_ids: str) -> Dict:
        """Pause streams in BM custom streams project"""
        return self._request('PUT', f'/bm-cs-projects/{project_id}/streams/{stream_ids}/pause')
    
    def restart_bm_streams(self, project_id: int, stream_ids: str) -> Dict:
        """Restart streams in BM custom streams project"""
        return self._request('PUT', f'/bm-cs-projects/{project_id}/streams/{stream_ids}/restart')

    # ==================== BM Custom Streams Data ====================
    
    def get_bm_stream_state(self, project_id: int, stream_id: str,
                            timeoffset: Optional[int] = None,
                            start_date: Optional[str] = None,
                            end_date: Optional[str] = None) -> Dict:
        """Get the state of the stream"""
        params = {}
        if timeoffset is not None:
            params['timeoffset'] = timeoffset
        if start_date:
            params['start_date'] = start_date
        if end_date:
            params['end_date'] = end_date
        return self._request('GET', f'/bm-cs-projects/{project_id}/streams/{stream_id}/state', params=params)
    
    def get_bm_stream_results(self, project_id: int, stream_id: str,
                              result_type: str = 'day',
                              date: Optional[str] = None,
                              min_duration: Optional[int] = None,
                              max_duration: Optional[int] = None,
                              isrc_country: Optional[str] = None,
                              with_false_positive: Optional[int] = None) -> Dict:
        """Get the stream monitoring results"""
        params = {'type': result_type}
        if date:
            params['date'] = date
        if min_duration is not None:
            params['min_duration'] = min_duration
        if max_duration is not None:
            params['max_duration'] = max_duration
        if isrc_country:
            params['isrc_country'] = isrc_country
        if with_false_positive is not None:
            params['with_false_positive'] = with_false_positive
        return self._request('GET', f'/bm-cs-projects/{project_id}/streams/{stream_id}/results', params=params)
    
    def get_bm_analytics(self, project_id: int, stats_type: str, result_type: str) -> Dict:
        """Get analytics data for the last 7 days"""
        params = {
            'stats_type': stats_type,
            'result_type': result_type
        }
        return self._request('GET', f'/bm-cs-projects/{project_id}/analytics/results', params=params)
    
    def add_bm_stream_user_report(self, project_id: int, stream_id: str, data: List[Dict]) -> Dict:
        """Insert user results"""
        payload = {'data': data}
        return self._request('POST', f'/bm-cs-projects/{project_id}/streams/{stream_id}/user-reports', json=payload)
    
    def get_bm_stream_recording(self, project_id: int, stream_id: str,
                                timestamp_utc: str, played_duration: int,
                                record_before: int = 0, record_after: int = 0) -> Dict:
        """Get the recording of the results"""
        params = {
            'timestamp_utc': timestamp_utc,
            'played_duration': played_duration,
            'record_before': record_before,
            'record_after': record_after
        }
        return self._request('GET', f'/bm-cs-projects/{project_id}/streams/{stream_id}/recordings', params=params)

    # ==================== BM Database Projects ====================
    
    def list_bm_bd_projects(self, region: Optional[str] = None) -> Dict:
        """List all BM Database projects"""
        params = {}
        if region:
            params['region'] = region
        return self._request('GET', '/bm-bd-projects', params=params)

    def create_bm_bd_project(self, name: str, region: str, buckets: List[int]) -> Dict:
        """Create a new BM Database project"""
        payload = {
            'name': name,
            'region': region,
            'buckets': buckets
        }
        return self._request('POST', '/bm-bd-projects', json=payload)

    def update_bm_bd_project(self, project_id: int, name: Optional[str] = None, buckets: Optional[List[int]] = None) -> Dict:
        """Update a BM Database project"""
        payload = {}
        if name:
            payload['name'] = name
        if buckets:
            payload['buckets'] = buckets
        return self._request('PUT', f'/bm-bd-projects/{project_id}', json=payload)

    def delete_bm_bd_project(self, project_id: int) -> Dict:
        """Delete a BM Database project"""
        return self._request('DELETE', f'/bm-bd-projects/{project_id}')

    def set_bm_bd_result_callback(self, project_id: int, result_callback_url: str,
                                  result_callback_send_noresult: int = 0,
                                  result_callback_result_type: int = 0) -> Dict:
        """Set result callback URL for BM Database project"""
        payload = {
            'result_callback_url': result_callback_url,
            'result_callback_send_noresult': result_callback_send_noresult,
            'result_callback_result_type': result_callback_result_type
        }
        return self._request('POST', f'/bm-bd-projects/{project_id}/result-callback', json=payload)

    def set_bm_bd_state_notification_callback(self, project_id: int, state_callback_url: str) -> Dict:
        """Set state notification callback for BM Database project"""
        payload = {
            'state_callback_url': state_callback_url
        }
        return self._request('POST', f'/bm-bd-projects/{project_id}/state-notification', json=payload)

    # ==================== BM Database Channels ====================
    
    def list_bm_bd_channels(self, project_id: int, state: str = 'All', timemap: Optional[str] = None,
                            search_type: Optional[str] = None, search_value: Optional[str] = None,
                            page: int = 1) -> Dict:
        """List channels in BM Database project"""
        params = {'state': state, 'page': page}
        if timemap:
            params['timemap'] = timemap
        if search_type and search_value:
            params['search_type'] = search_type
            params['search_value'] = search_value
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels', params=params)

    def add_bm_bd_channels(self, project_id: int, channels: List[int]) -> Dict:
        """Add channels to BM Database project"""
        payload = {'channels': channels}
        return self._request('POST', f'/bm-bd-projects/{project_id}/channels', json=payload)

    def delete_bm_bd_channels(self, project_id: int, channel_ids: str) -> Dict:
        """Delete channels from BM Database project"""
        return self._request('DELETE', f'/bm-bd-projects/{project_id}/channels/{channel_ids}')

    def set_bm_bd_channel_custom_id(self, project_id: int, channel_id: int, custom_id: str) -> Dict:
        """Set custom_id for a channel"""
        params = {'custom_id': custom_id}
        return self._request('POST', f'/bm-bd-projects/{project_id}/channels/{channel_id}/user-defined', params=params)

    # ==================== BM Database Channels Data ====================
    
    def get_bm_bd_channel_results(self, project_id: int, channel_id: int, result_type: str = 'day',
                                  date: Optional[str] = None, min_duration: Optional[int] = None,
                                  max_duration: Optional[int] = None, isrc_country: Optional[str] = None,
                                  with_false_positive: Optional[int] = None) -> Dict:
        """Get non-real-time results of channel monitoring"""
        params = {'type': result_type}
        if date:
            params['date'] = date
        if min_duration is not None:
            params['min_duration'] = min_duration
        if max_duration is not None:
            params['max_duration'] = max_duration
        if isrc_country:
            params['isrc_country'] = isrc_country
        if with_false_positive is not None:
            params['with_false_positive'] = with_false_positive
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels/{channel_id}/results', params=params)

    def get_bm_bd_channel_unknown_results(self, project_id: int, channel_id: int,
                                          date: str, min_duration: Optional[int] = None,
                                          max_duration: Optional[int] = None) -> Dict:
        """Get unknown music results of channel monitoring"""
        params = {'date': date}
        if min_duration is not None:
            params['min_duration'] = min_duration
        if max_duration is not None:
            params['max_duration'] = max_duration
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels/{channel_id}/unknown_results', params=params)

    def get_bm_bd_channel_realtime_results(self, project_id: int, channel_id: int) -> Dict:
        """Get real-time results of channel monitoring"""
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels/{channel_id}/realtime_results')

    def get_bm_bd_channel_state(self, project_id: int, channel_id: int,
                                timeoffset: Optional[int] = None,
                                start_date: Optional[str] = None,
                                end_date: Optional[str] = None) -> Dict:
        """Get the state of the channel"""
        params = {}
        if timeoffset is not None:
            params['timeoffset'] = timeoffset
        if start_date:
            params['start_date'] = start_date
        if end_date:
            params['end_date'] = end_date
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels/{channel_id}/state', params=params)

    def get_bm_bd_analytics(self, project_id: int, stats_type: str, result_type: str) -> Dict:
        """Get analytics data for the last 7 days"""
        params = {
            'stats_type': stats_type,
            'result_type': result_type
        }
        return self._request('GET', f'/bm-bd-projects/{project_id}/analytics/results', params=params)

    def add_bm_bd_channel_user_report(self, project_id: int, channel_id: int, data: List[Dict]) -> Dict:
        """Insert user results"""
        payload = {'data': data}
        return self._request('POST', f'/bm-bd-projects/{project_id}/channels/{channel_id}/user-reports', json=payload)

    def get_bm_bd_channel_recording(self, project_id: int, channel_id: int,
                                    timestamp_utc: str, played_duration: int,
                                    record_before: int = 0, record_after: int = 0) -> Dict:
        """Get the recording of the results"""
        params = {
            'timestamp_utc': timestamp_utc,
            'played_duration': played_duration,
            'record_before': record_before,
            'record_after': record_after
        }
        return self._request('GET', f'/bm-bd-projects/{project_id}/channels/{channel_id}/recordings', params=params)

    # ==================== UCF Projects ====================

    def list_ucf_projects(self, region: Optional[str] = None) -> Dict:
        """List all UCF projects"""
        params = {}
        if region:
            params['region'] = region
        return self._request('GET', '/ucf-projects', params=params)

    def create_ucf_project(self, name: str, region: str, project_type: str = 'BM',
                           config: Optional[Dict] = None) -> Dict:
        """Create a new UCF project"""
        payload = {
            'name': name,
            'region': region,
            'type': project_type
        }
        if config:
            payload['config'] = config
        return self._request('POST', '/ucf-projects', json=payload)

    def update_ucf_project(self, project_id: int, name: Optional[str] = None,
                           config: Optional[Dict] = None) -> Dict:
        """Update a UCF project"""
        payload = {}
        if name:
            payload['name'] = name
        if config:
            payload['config'] = config
        return self._request('PUT', f'/ucf-projects/{project_id}', json=payload)

    def delete_ucf_project(self, project_id: int) -> Dict:
        """Delete a UCF project"""
        return self._request('DELETE', f'/ucf-projects/{project_id}')

    # ==================== UCF BM Streams ====================

    def list_ucf_streams(self, project_id: int, page: int = 1, per_page: int = 10) -> Dict:
        """List UCF BM streams"""
        params = {'page': page, 'per_page': per_page}
        return self._request('GET', f'/ucf-projects/{project_id}/streams', params=params)

    def import_ucf_bm_streams(self, project_id: int, bm_stream_ids: List[str],
                              origin_from: str, bm_project_id: int) -> Dict:
        """Import BM streams to UCF project"""
        payload = {
            'bm_stream_ids': bm_stream_ids,
            'from': origin_from,
            'bm_project_id': bm_project_id
        }
        return self._request('POST', f'/ucf-projects/{project_id}/streams', json=payload)

    def delete_ucf_bm_streams(self, project_id: int, stream_ids: str) -> Dict:
        """Delete UCF BM streams"""
        return self._request('DELETE', f'/ucf-projects/{project_id}/streams/{stream_ids}')

    # ==================== UCF Results ====================

    def list_ucf_results(self, project_id: int, page: int = 1, per_page: int = 10,
                         begin_date: Optional[str] = None, end_date: Optional[str] = None,
                         sortby: Optional[str] = None, order: Optional[str] = None,
                         status: Optional[str] = None, min_duration: Optional[str] = None,
                         max_duration: Optional[str] = None, streams: Optional[str] = None,
                         ucf_id: Optional[str] = None, label: Optional[str] = None,
                         label_value: Optional[str] = None) -> Dict:
        """List UCF results"""
        params = {'page': page, 'per_page': per_page}
        if begin_date: params['begin_date'] = begin_date
        if end_date: params['end_date'] = end_date
        if sortby: params['sortby'] = sortby
        if order: params['order'] = order
        if status: params['status'] = status
        if min_duration: params['min'] = min_duration
        if max_duration: params['max'] = max_duration
        if streams: params['streams'] = streams
        if ucf_id: params['ucf_id'] = ucf_id
        if label: params['label'] = label
        if label_value: params['label_value'] = label_value
        return self._request('GET', f'/ucf-projects/{project_id}/results', params=params)

    def get_ucf_result(self, project_id: int, ucf_id: str) -> Dict:
        """Get one UCF item"""
        return self._request('GET', f'/ucf-projects/{project_id}/results/{ucf_id}')

    def get_ucf_result_details(self, project_id: int, ucf_id: int, page: int = 1,
                               per_page: int = 10, begin_date: Optional[str] = None,
                               end_date: Optional[str] = None) -> Dict:
        """Get UCF item details"""
        params = {'page': page, 'per_page': per_page}
        if begin_date: params['begin_date'] = begin_date
        if end_date: params['end_date'] = end_date
        return self._request('GET', f'/ucf-projects/{project_id}/results/{ucf_id}/details', params=params)

    def get_ucf_record_url(self, project_id: int, ucf_id: str, extend: int = 20) -> Dict:
        """Get UCF record URL"""
        params = {'extend': extend}
        return self._request('GET', f'/ucf-projects/{project_id}/results/{ucf_id}/record', params=params)

    def delete_ucf_item(self, project_id: int, ucf_id: int, reserved: int = 0) -> Dict:
        """Delete or reserve a UCF item"""
        params = {'reserved': reserved}
        return self._request('DELETE', f'/ucf-projects/{project_id}/results/{ucf_id}', params=params)

    def set_ucf_item_pending(self, project_id: int, ucf_id: int) -> Dict:
        """Make a UCF item status pending"""
        return self._request('POST', f'/ucf-projects/{project_id}/results/{ucf_id}/pending')

    # ==================== Billing ====================

    def _download_request(self, endpoint: str) -> bytes:
        """Make HTTP request and return raw bytes (for file downloads)"""
        url = f"{self.base_url}{endpoint}"
        response = self.session.request('GET', url)
        try:
            response.raise_for_status()
            return response.content
        except requests.exceptions.HTTPError as e:
            error_msg = f"HTTP Error {response.status_code}"
            try:
                error_data = response.json()
                if 'message' in error_data:
                    error_msg = f"{error_msg}: {error_data['message']}"
                elif 'error' in error_data:
                    error_msg = f"{error_msg}: {error_data['error']}"
            except Exception:
                error_msg = f"{error_msg}: {response.text}"
            raise APIError(error_msg, response.status_code)

    def get_current_bill(self, uid: Optional[int] = None) -> Dict:
        """Get the current/latest bill for the authenticated user"""
        params = {}
        if uid is not None:
            params['uid'] = uid
        return self._request('GET', '/billing/current-bill', params=params)

    def get_next_bill_date(self, uid: Optional[int] = None) -> Dict:
        """Get the next billing date and remaining days"""
        params = {}
        if uid is not None:
            params['uid'] = uid
        return self._request('GET', '/billing/next-bill-date', params=params)

    def list_invoices(self, uid: Optional[int] = None, per_page: int = 20) -> Dict:
        """Get paginated list of invoices"""
        params = {'per_page': per_page}
        if uid is not None:
            params['uid'] = uid
        return self._request('GET', '/billing/invoices', params=params)

    def download_invoice(self, invoice_id: str) -> bytes:
        """Download a specific invoice as PDF bytes"""
        return self._download_request(f'/billing/invoices/{invoice_id}/download')

    def get_prices(self, uid: Optional[int] = None,
                   price_type: Optional[str] = None,
                   service_types: Optional[str] = None) -> Dict:
        """Get pricing information for services"""
        params = {}
        if uid is not None:
            params['uid'] = uid
        if price_type:
            params['type'] = price_type
        if service_types:
            params['service_types'] = service_types
        return self._request('GET', '/prices', params=params)


class APIError(Exception):
    """API Error exception"""
    def __init__(self, message: str, status_code: int = None):
        super().__init__(message)
        self.status_code = status_code
        self.message = message



"""
Utility functions for ACRCloud CLI
"""

import click
import json
from typing import List, Any


def output_json(data: dict, pretty: bool = True):
    """Output data as formatted JSON"""
    if pretty:
        click.echo(json.dumps(data, indent=2, ensure_ascii=False))
    else:
        click.echo(json.dumps(data, ensure_ascii=False))


def output_table(headers: List[str], rows: List[List[Any]]):
    """Output data as a formatted table"""
    if not rows:
        click.echo("No data to display")
        return
    
    # Calculate column widths
    col_widths = []
    for i, header in enumerate(headers):
        max_width = len(str(header))
        for row in rows:
            if i < len(row):
                max_width = max(max_width, len(str(row[i])))
        col_widths.append(max_width + 2)  # Add padding
    
    # Print header
    header_line = ""
    for i, header in enumerate(headers):
        header_line += str(header).ljust(col_widths[i])
    click.echo(header_line)
    click.echo("-" * sum(col_widths))
    
    # Print rows
    for row in rows:
        row_line = ""
        for i, cell in enumerate(row):
            if i < len(col_widths):
                row_line += str(cell).ljust(col_widths[i])
        click.echo(row_line)


def confirm_action(message: str) -> bool:
    """Ask user for confirmation"""
    return click.confirm(message, default=False)


def format_bytes(size_bytes: int) -> str:
    """Format bytes to human-readable string"""
    if size_bytes == 0:
        return "0 B"
    
    size_names = ["B", "KB", "MB", "GB", "TB"]
    i = 0
    while size_bytes >= 1024 and i < len(size_names) - 1:
        size_bytes /= 1024
        i += 1
    
    return f"{size_bytes:.2f} {size_names[i]}"


def parse_metadata_template(template_str: str) -> dict:
    """Parse metadata template string to dict"""
    try:
        return json.loads(template_str)
    except json.JSONDecodeError as e:
        raise click.BadParameter(f"Invalid JSON: {e}")


def validate_region(ctx, param, value):
    """Validate region parameter"""
    valid_regions = ['eu-west-1', 'us-west-2', 'ap-southeast-1']
    if value and value not in valid_regions:
        raise click.BadParameter(f"Invalid region. Must be one of: {', '.join(valid_regions)}")
    return value


def validate_bucket_type(ctx, param, value):
    """Validate bucket type parameter"""
    valid_types = ['File', 'Live', 'LiveRec', 'LiveTimeshift']
    if value and value not in valid_types:
        raise click.BadParameter(f"Invalid bucket type. Must be one of: {', '.join(valid_types)}")
    return value



"""
Billing & Pricing commands for ACRCloud CLI
"""

import os
import click
from ..api import ACRCloudAPI
from ..utils import output_json, output_table


@click.group(name='billing')
@click.pass_context
def billing(ctx):
    """Manage billing, invoices, and pricing"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )


@billing.command(name='current-bill')
@click.option('--uid', type=int, default=None, help='User ID (admin only)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def current_bill(ctx, uid, output):
    """Get the current/latest bill

    Examples:
        acrcloud billing current-bill
        acrcloud billing current-bill --uid 12345
        acrcloud billing current-bill --output json
    """
    api = ctx.obj['api']

    try:
        result = api.get_current_bill(uid=uid)

        if output == 'json':
            output_json(result)
        else:
            bill = result.get('data', {})
            if not bill:
                click.echo("No current bill found")
                return

            state_map = {0: 'Pending', 1: 'Paid', 2: 'Cancelled'}
            click.echo("Current Bill:")
            click.echo("-" * 40)
            click.echo(f"  Bill ID:        {bill.get('id')}")
            click.echo(f"  User ID:        {bill.get('uid')}")
            click.echo(f"  Charge Type:    {bill.get('charge_type')}")
            click.echo(f"  Amount:         {bill.get('amount')}")
            click.echo(f"  Credit:         {bill.get('credit')}")
            click.echo(f"  Billing Period: {bill.get('bill_begin')} ~ {bill.get('bill_end')}")
            click.echo(f"  State:          {state_map.get(bill.get('state'), bill.get('state'))}")
            click.echo(f"  Payment Method: {bill.get('payment_method')}")
            click.echo(f"  Created:        {bill.get('created_at')}")
            click.echo(f"  Updated:        {bill.get('updated_at')}")

    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@billing.command(name='next-bill-date')
@click.option('--uid', type=int, default=None, help='User ID (admin only)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def next_bill_date(ctx, uid, output):
    """Get the next billing date and days remaining

    Examples:
        acrcloud billing next-bill-date
        acrcloud billing next-bill-date --uid 12345
        acrcloud billing next-bill-date --output json
    """
    api = ctx.obj['api']

    try:
        result = api.get_next_bill_date(uid=uid)

        if output == 'json':
            output_json(result)
        else:
            click.echo("Next Bill Date:")
            click.echo("-" * 40)
            click.echo(f"  Next Bill Date: {result.get('next_bill_date')}")
            click.echo(f"  Days Remaining: {result.get('left_days')}")

    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@billing.command(name='invoices')
@click.option('--uid', type=int, default=None, help='User ID (admin only)')
@click.option('--per-page', '-n', default=20, help='Results per page')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_invoices(ctx, uid, per_page, output):
    """List invoices

    Examples:
        acrcloud billing invoices
        acrcloud billing invoices --uid 12345 --per-page 50
        acrcloud billing invoices --output json
    """
    api = ctx.obj['api']

    try:
        result = api.list_invoices(uid=uid, per_page=per_page)

        if output == 'json':
            output_json(result)
        else:
            invoices = result.get('data', [])
            if not invoices:
                click.echo("No invoices found")
                return

            # Pagination info
            total = result.get('total')
            current_page = result.get('current_page')
            last_page = result.get('last_page')
            if total is not None:
                click.echo(f"Showing page {current_page}/{last_page}, total: {total}")

            state_map = {0: 'Pending', 1: 'Paid', 2: 'Cancelled'}
            headers = ['ID', 'Charge Type', 'Amount', 'Period', 'State', 'Payment Method']
            rows = []
            for inv in invoices:
                period = f"{inv.get('bill_begin')} ~ {inv.get('bill_end')}"
                rows.append([
                    inv.get('id'),
                    inv.get('charge_type'),
                    inv.get('amount'),
                    period,
                    state_map.get(inv.get('state'), inv.get('state')),
                    inv.get('payment_method', '-'),
                ])
            output_table(headers, rows)

    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@billing.command(name='download-invoice')
@click.argument('invoice_id')
@click.option('--output-file', '-f', default=None,
              help='Output file path (default: <invoice_id>.pdf)')
@click.pass_context
def download_invoice(ctx, invoice_id, output_file):
    """Download an invoice as PDF

    INVOICE_ID should be in the format {uid}-{bill_id}, e.g. 12345-67890

    Examples:
        acrcloud billing download-invoice 12345-67890
        acrcloud billing download-invoice 12345-67890 --output-file ~/invoices/my_invoice.pdf
    """
    api = ctx.obj['api']

    if not output_file:
        output_file = f"{invoice_id}.pdf"

    try:
        pdf_bytes = api.download_invoice(invoice_id)
        output_path = os.path.expanduser(output_file)
        with open(output_path, 'wb') as f:
            f.write(pdf_bytes)
        click.echo(f"Invoice saved to: {output_path}")

    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@billing.command(name='prices')
@click.option('--uid', type=int, default=None, help='User ID (admin only)')
@click.option('--type', 'price_type',
              type=click.Choice(['Standard', 'All', 'Latest']),
              default=None,
              help='Price type filter (Standard/All/Latest, default: Latest)')
@click.option('--service-types', default=None,
              help='Filter by service types, comma-separated (e.g. AVR,LCD,BM)')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def get_prices(ctx, uid, price_type, service_types, output):
    """Get pricing information for services

    Examples:
        acrcloud billing prices
        acrcloud billing prices --type Standard
        acrcloud billing prices --service-types AVR,LCD
        acrcloud billing prices --uid 12345 --type All --output json
    """
    api = ctx.obj['api']

    try:
        result = api.get_prices(uid=uid, price_type=price_type, service_types=service_types)

        if output == 'json':
            output_json(result)
        else:
            prices_data = result.get('data', [])
            discount = result.get('discount')

            if not prices_data:
                click.echo("No pricing data found")
                return

            headers = ['ID', 'UID', 'Service Type', 'State', 'Prices']
            rows = []
            for price in prices_data:
                prices_obj = price.get('prices', {})
                prices_str = ', '.join(f"{k}: {v}" for k, v in prices_obj.items()) if prices_obj else '-'
                rows.append([
                    price.get('id'),
                    price.get('uid'),
                    price.get('service_type'),
                    'Active' if price.get('state') == 1 else 'Inactive',
                    prices_str,
                ])
            output_table(headers, rows)

            if discount:
                click.echo()
                click.echo("Your Discount:")
                for k, v in discount.items():
                    click.echo(f"  {k}: {v}")

    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()



"""
Projects command for ACRCloud CLI
"""

import click
import json
from ..api import ACRCloudAPI
from ..utils import output_json, output_table, confirm_action


@click.group(name='base-projects')
@click.pass_context
def projects(ctx):
    """Manage ACRCloud recognition projects"""
    ctx.obj['api'] = ACRCloudAPI(
        ctx.obj['access_token'],
        ctx.obj['base_url']
    )


@projects.command(name='list')
@click.option('--page', '-p', default=1, help='Page number')
@click.option('--per-page', '-n', default=20, help='Results per page')
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def list_projects(ctx, page, per_page, output):
    """List all projects
    
    Examples:
        acrcloud base-projects list
        acrcloud base-projects list --output json
    """
    api = ctx.obj['api']
    
    try:
        result = api.list_projects(page=page, per_page=per_page)
        
        if output == 'json':
            output_json(result)
        else:
            projects_data = result.get('data', [])
            if not projects_data:
                click.echo("No projects found")
                return
            
            headers = ['ID', 'Name', 'Type', 'Region', 'Audio Type', 'State']
            rows = []
            for project in projects_data:
                rows.append([
                    project.get('id'),
                    project.get('name', '-')[:25],
                    project.get('service_type', '-'),
                    project.get('region', '-'),
                    project.get('audio_type', '-'),
                    'Active' if project.get('state') == 1 else 'Inactive'
                ])
            output_table(headers, rows)
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='get')
@click.argument('project_id', type=int)
@click.option('--output', '-o', type=click.Choice(['json', 'table']), default='table', help='Output format')
@click.pass_context
def get_project(ctx, project_id, output):
    """Get project details
    
    Examples:
        acrcloud base-projects get 12345
        acrcloud base-projects get 12345 --output json
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_project(project_id)
        
        if output == 'json':
            output_json(result)
        else:
            project = result.get('data', {})
            if not project:
                click.echo("Project not found")
                return
            
            click.echo(f"Project Details (ID: {project_id}):")
            click.echo("-" * 40)
            click.echo(f"  Name: {project.get('name')}")
            click.echo(f"  Type: {project.get('service_type')}")
            click.echo(f"  Region: {project.get('region')}")
            click.echo(f"  Audio Type: {project.get('audio_type')}")
            click.echo(f"  Access Key: {project.get('access_key')}")
            click.echo(f"  Access Secret: {'*' * 20}")
            click.echo(f"  Buckets: {', '.join(map(str, project.get('buckets', [])))}")
            click.echo(f"  State: {'Active' if project.get('state') == 1 else 'Inactive'}")
            click.echo(f"  Created: {project.get('created_at')}")
            click.echo(f"  Updated: {project.get('updated_at')}")
            
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='create')
@click.option('--name', '-n', required=True, help='Project name')
@click.option('--type', '-t', 'project_type', required=True,
              type=click.Choice(['AVR', 'LCD', 'HR']),
              help='Project type (AVR=audio/video recognition, LCD=live channel detection, HR=hybrid)')
@click.option('--region', '-r', required=True,
              type=click.Choice(['eu-west-1', 'us-west-2', 'ap-southeast-1']),
              help='Region')
@click.option('--buckets', '-b', default='23', help='Comma-separated bucket IDs')
@click.option('--audio-type', type=click.Choice(['linein', 'recorded']), default='linein',
              help='Audio type (linein=original, recorded=microphone)')
@click.option('--external-ids', '-e', help='External IDs (spotify,deezer,isrc,upc,musicbrainz)')
@click.pass_context
def create_project(ctx, name, project_type, region, buckets, audio_type, external_ids):
    """Create a new recognition project
    
    Examples:
        acrcloud base-projects create --name my-project --type AVR --region eu-west-1 --buckets "1,2,3"
        acrcloud base-projects create -n music-detection -t AVR -r ap-southeast-1 -b "12345" --audio-type linein
    """
    api = ctx.obj['api']
    
    bucket_ids = [int(b.strip()) for b in buckets.split(',')]
    
    try:
        result = api.create_project(
            name=name,
            project_type=project_type,
            region=region,
            buckets=bucket_ids,
            audio_type=audio_type,
            external_ids=external_ids
        )
        
        project = result.get('data', {})
        click.echo(f"✓ Project created successfully!")
        click.echo(f"  ID: {project.get('id')}")
        click.echo(f"  Name: {project.get('name')}")
        click.echo(f"  Type: {project.get('service_type')}")
        click.echo(f"  Access Key: {project.get('access_key')}")
        click.echo(f"  Access Secret: {project.get('access_secret')}")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='update')
@click.argument('project_id', type=int)
@click.option('--name', '-n', help='New project name')
@click.option('--buckets', '-b', help='Comma-separated bucket IDs')
@click.option('--audio-type', type=click.Choice(['linein', 'recorded']), help='Audio type')
@click.pass_context
def update_project(ctx, project_id, name, buckets, audio_type):
    """Update a project
    
    Examples:
        acrcloud base-projects update 12345 --name new-name
        acrcloud base-projects update 12345 -b "1,2,3,4"
    """
    api = ctx.obj['api']
    
    bucket_ids = [int(b.strip()) for b in buckets.split(',')] if buckets else None
    
    try:
        result = api.update_project(
            project_id=project_id,
            name=name,
            buckets=bucket_ids,
            audio_type=audio_type
        )
        
        click.echo(f"✓ Project {project_id} updated successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='delete')
@click.argument('project_id', type=int)
@click.option('--yes', '-y', is_flag=True, help='Skip confirmation')
@click.pass_context
def delete_project(ctx, project_id, yes):
    """Delete a project
    
    Examples:
        acrcloud base-projects delete 12345
        acrcloud base-projects delete 12345 --yes
    """
    api = ctx.obj['api']
    
    if not yes:
        if not confirm_action(f"Are you sure you want to delete project {project_id}?"):
            click.echo("Cancelled")
            return
    
    try:
        api.delete_project(project_id)
        click.echo(f"✓ Project {project_id} deleted successfully!")
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='bucket-status')
@click.argument('project_id', type=int)
@click.pass_context
def bucket_status(ctx, project_id):
    """Get the status of project's buckets
    
    Examples:
        acrcloud base-projects bucket-status 12345
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_project_bucket_status(project_id)
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()


@projects.command(name='statistics')
@click.argument('project_id', type=int)
@click.option('--start-date', '-s', help='Start date (YYYY-MM-DD)')
@click.option('--end-date', '-e', help='End date (YYYY-MM-DD)')
@click.pass_context
def project_statistics(ctx, project_id, start_date, end_date):
    """Get project statistics
    
    Examples:
        acrcloud base-projects statistics 12345
        acrcloud base-projects statistics 12345 --start-date 2024-01-01 --end-date 2024-12-31
    """
    api = ctx.obj['api']
    
    try:
        result = api.get_project_statistics(
            project_id=project_id,
            start_date=start_date,
            end_date=end_date
        )
        output_json(result)
        
    except Exception as e:
        click.echo(f"Error: {e}", err=True)
        raise click.Abort()
